/* overview.c - info about all sections contained in ELF files
   This file is loosely based on readelf.c from the Red Hat elfutils
   Copyright (C) 2006 Christian Aichinger <Greek0@gmx.net>
   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006 Red Hat, Inc.

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
   Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
   details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
   */

#include <argp.h>
#include <gelf.h>
#include <inttypes.h>
#include <libelf.h>
#include <elfutils/libebl.h>
#include <libintl.h>
#include <locale.h>
#include <mcheck.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdio_ext.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/param.h>

#include "utils.h"
#include "commands.h"
#include "output.h"

void show_overview (UElf_File *efile) {
    /* Iterate over all sections.  */
    Elf_Scn *scn = NULL;
    while ((scn = elf_nextscn (efile->elf, scn)) != NULL) {
        GElf_Shdr shdr_mem;
        GElf_Shdr *shdr = gelf_getshdr (scn, &shdr_mem);
  
        if (shdr == NULL)
            internal_error("Couldn't get shdr for '%s'\n", efile->fname);
  
        print_section(efile, elf_ndxscn(scn), shdr);
    }
}

