/* utils.h - prototypes for the UElf stuff, a convenient wrapper around libelf
   Copyright (C) 2006 Christian Aichinger <Greek0@gmx.net>

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
   Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
   details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
   */

#ifndef __SEEN_UTILS_H
#define __SEEN_UTILS_H

#define SIF_SYMBOL    1
#define SIF_SEC_VALID 2

#include "sparse_array.h"

typedef struct UElf_File {
    Elf* elf;
    Ebl* ebl;
    size_t shnum;
    size_t shstrndx;
    const char* fname;
    const char **scnnames;
} UElf_File;

typedef struct UELf_Symbol {
    uint32_t index;
    uint32_t flags;
    unsigned char type;
    unsigned char bind;
    const char *name;
    GElf_Sym sym;
} UElf_Symbol;

typedef struct UElf_Section {
    Elf_Scn *scn;
    GElf_Shdr smem;
    GElf_Shdr *shdr;
    Elf_Data *data;
} UElf_Section;
const UElf_Section UElf_Section_INIT;

#define VMT_UNKNOWN 0
#define VMT_VERDEF  1
#define VMT_VERNEED 2
typedef struct UElf_VersionMeta {
    const char *vername;
    const char *filename;

    unsigned char type;
    union {
        struct {
            Elf32_Verneed vn;
            Elf32_Vernaux vna;
        } need;
        struct {
            Elf32_Verdef  vd;
            Elf32_Verdaux vda;
        } def;
    };
} UElf_VersionMeta;

typedef struct UElf_VersionInfo {
    UElf_Section sec_sym;
    UElf_Section sec_def;
    UElf_Section sec_need;
    UElf_Section sec_strings;

    sarray meta;
} UElf_VersionInfo;

typedef struct UElf_SymVersion {
    GElf_Versym symmem;
    GElf_Versym *sym;

    GElf_Half hidden;
    GElf_Half index;
    const UElf_VersionMeta *meta;
} UElf_SymVersion;

UElf_File* file_new(Elf* elf, Ebl* ebl, const char* fname);
void       file_free(UElf_File *efile);

UElf_VersionInfo* verinfo_new(UElf_File *efile);
void              verinfo_free(UElf_VersionInfo *verinfo);

void resolve_symbol(UElf_File *efile, UElf_Section *section, int ndx, UElf_Symbol *result);

bool section_find(UElf_File *efile, GElf_Word sh_type, UElf_Section* section);
bool section_from_index(UElf_File *efile, GElf_Word index, UElf_Section* section);
bool section_iterate(UElf_File *efile, UElf_Section* section);
void section_fill_data(UElf_File *efile, UElf_Section* section);

void versym_fill(UElf_File *efile, UElf_VersionInfo* verinfo, GElf_Word index, UElf_SymVersion *sv);

#endif // __SEEN_UTILS_H
