/* versioniter.c - deep magic for iterating over verdef/verneed entries
                   codename: "submit me for next years IOCCC"
   This file is loosely based on readelf.c from the Red Hat elfutils
   Copyright (C) 2006 Christian Aichinger <Greek0@gmx.net>
   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006 Red Hat, Inc.

   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
   Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
   details.

   You should have received a copy of the GNU General Public License along with
   this program; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
   */

#include <argp.h>
#include <fcntl.h>
#include <gelf.h>
#include <inttypes.h>
#include <libelf.h>
#include <elfutils/libebl.h>
#include <libintl.h>
#include <locale.h>
#include <mcheck.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdio_ext.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/param.h>

#include "utils.h"
#include "versioniter.h"

const UElf_Verdef_Iter UElf_Verdef_Iter_INIT = { 0 };
const UElf_Verneed_Iter UElf_Verneed_Iter_INIT = { 0 };

#define generic_iter_start(cnt, maxcnt, offset)                         \
    if (iter->cnt >= maxcnt)                                            \
        return false;                                                   \
    iter->offset = iter->_ ## offset;

#define generic_iter_get_value(name, function, offset, memory)          \
    iter->name = function(section->data, offset, memory);               \
    if (!iter->name)                                                    \
        return false;

#define generic_iter_zero_aux(name, count, offset, storeoffset)         \
    iter->name = NULL; iter->count = 0;                                 \
    iter->offset = iter->_ ## offset = storeoffset;

#define generic_iter_end(cnt, maxcnt, offset, offsetinc)                \
    iter->_ ## offset += offsetinc;                                     \
    iter->cnt++;                                                        \
    return true;

bool iter_verdef(UElf_File *efile, UElf_Section *section, UElf_Verdef_Iter *iter) {
    generic_iter_start(count, section->shdr->sh_info, offset);
    generic_iter_get_value(def, gelf_getverdef, iter->offset, &iter->defmem);
    generic_iter_zero_aux(aux, auxcount, auxoffset, iter->offset + iter->def->vd_aux);
    generic_iter_end(count, section->shdr->sh_info, offset, iter->def->vd_next);
}

bool iter_verdefaux(UElf_File *efile, UElf_Section *section, UElf_Verdef_Iter *iter) {
    generic_iter_start(auxcount, iter->def->vd_cnt, auxoffset);
    generic_iter_get_value(aux, gelf_getverdaux, iter->auxoffset, &iter->auxmem);
    generic_iter_end(auxcount, iter->def->vd_cnt, auxoffset, iter->aux->vda_next);
}
    
bool iter_verneed(UElf_File *efile, UElf_Section *section, UElf_Verneed_Iter *iter) {
    generic_iter_start(count, section->shdr->sh_info, offset);
    generic_iter_get_value(need, gelf_getverneed, iter->offset, &iter->needmem);
    generic_iter_zero_aux(aux, auxcount, auxoffset, iter->offset + iter->need->vn_aux);
    generic_iter_end(count, section->shdr->sh_info, offset, iter->need->vn_next);
}

bool iter_verneedaux(UElf_File *efile, UElf_Section *section, UElf_Verneed_Iter *iter) {
    generic_iter_start(auxcount, iter->need->vn_cnt, auxoffset);
    generic_iter_get_value(aux, gelf_getvernaux, iter->auxoffset, &iter->auxmem);
    generic_iter_end(auxcount, iter->need->vn_cnt, auxoffset, iter->aux->vna_next);
}

